@@ -5,7 +5,7 @@
  * @output wp-admin/js/inline-edit-post.js
  */
 
-/* global inlineEditL10n, ajaxurl, typenow, inlineEditPost */
+/* global ajaxurl, typenow, inlineEditPost */
 
 window.wp = window.wp || {};
 
@@ -19,7 +19,7 @@ window.wp = window.wp || {};
  * @type {Object}
  *
  * @property {string} type The type of inline editor.
- * @property {string} what The prefix before the post id.
+ * @property {string} what The prefix before the post ID.
  *
  */
 ( function( $, wp ) {
@@ -51,7 +51,7 @@ window.wp = window.wp || {};
 		 *
 		 * @return {boolean} The result of revert.
 		 */
-		qeRow.keyup(function(e){
+		qeRow.on( 'keyup', function(e){
 			// Revert changes if Escape key is pressed.
 			if ( e.which === 27 ) {
 				return inlineEditPost.revert();
@@ -63,7 +63,7 @@ window.wp = window.wp || {};
 		 *
 		 * @return {boolean} The result of revert.
 		 */
-		bulkRow.keyup(function(e){
+		bulkRow.on( 'keyup', function(e){
 			// Revert changes if Escape key is pressed.
 			if ( e.which === 27 ) {
 				return inlineEditPost.revert();
@@ -75,7 +75,7 @@ window.wp = window.wp || {};
 		 *
 		 * @return {boolean} The result of revert.
 		 */
-		$( '.cancel', qeRow ).click( function() {
+		$( '.cancel', qeRow ).on( 'click', function() {
 			return inlineEditPost.revert();
 		});
 
@@ -84,7 +84,7 @@ window.wp = window.wp || {};
 		 *
 		 * @return {boolean} The result of save.
 		 */
-		$( '.save', qeRow ).click( function() {
+		$( '.save', qeRow ).on( 'click', function() {
 			return inlineEditPost.save(this);
 		});
 
@@ -93,7 +93,7 @@ window.wp = window.wp || {};
 		 *
 		 * @return {boolean} The result of save.
 		 */
-		$('td', qeRow).keydown(function(e){
+		$('td', qeRow).on( 'keydown', function(e){
 			if ( e.which === 13 && ! $( e.target ).hasClass( 'cancel' ) ) {
 				return inlineEditPost.save(this);
 			}
@@ -104,14 +104,14 @@ window.wp = window.wp || {};
 		 *
 		 * @return {boolean} The result of revert.
 		 */
-		$( '.cancel', bulkRow ).click( function() {
+		$( '.cancel', bulkRow ).on( 'click', function() {
 			return inlineEditPost.revert();
 		});
 
 		/**
 		 * Disables the password input field when the private post checkbox is checked.
 		 */
-		$('#inline-edit .inline-edit-private input[value="private"]').click( function(){
+		$('#inline-edit .inline-edit-private input[value="private"]').on( 'click', function(){
 			var pw = $('input.inline-edit-password-input');
 			if ( $(this).prop('checked') ) {
 				pw.val('').prop('disabled', true);
@@ -131,7 +131,7 @@ window.wp = window.wp || {};
 		$('#bulk-edit').find('fieldset:first').after(
 			$('#inline-edit fieldset.inline-edit-categories').clone()
 		).siblings( 'fieldset:last' ).prepend(
-			$('#inline-edit label.inline-edit-tags').clone()
+			$( '#inline-edit .inline-edit-tags-wrap' ).clone()
 		);
 
 		$('select[name="_status"] option[value="future"]', bulkRow).remove();
@@ -139,7 +139,7 @@ window.wp = window.wp || {};
 		/**
 		 * Adds onclick events to the apply buttons.
 		 */
-		$('#doaction, #doaction2').click(function(e){
+		$('#doaction').on( 'click', function(e){
 			var n;
 
 			t.whichBulkButtonId = $( this ).attr( 'id' );
@@ -197,9 +197,15 @@ window.wp = window.wp || {};
 			// If the checkbox for a post is selected, add the post to the edit list.
 			if ( $(this).prop('checked') ) {
 				c = false;
-				var id = $(this).val(), theTitle;
-				theTitle = $('#inline_'+id+' .post_title').html() || inlineEditL10n.notitle;
-				te += '<div id="ttle'+id+'"><a id="_'+id+'" class="ntdelbutton" title="'+inlineEditL10n.ntdeltitle+'">X</a>'+theTitle+'</div>';
+				var id = $( this ).val(),
+					theTitle = $( '#inline_' + id + ' .post_title' ).html() || wp.i18n.__( '(no title)' ),
+					buttonVisuallyHiddenText = wp.i18n.sprintf(
+						/* translators: %s: Post title. */
+						wp.i18n.__( 'Remove &#8220;%s&#8221; from Bulk Edit' ),
+						theTitle
+					);
+
+				te += '<li class="ntdelitem"><button type="button" id="_' + id + '" class="button-link ntdelbutton"><span class="screen-reader-text">' + buttonVisuallyHiddenText + '</span></button><span class="ntdeltitle" aria-hidden="true">' + theTitle + '</span></li>';
 			}
 		});
 
@@ -208,18 +214,34 @@ window.wp = window.wp || {};
 			return this.revert();
 		}
 
-		// Add onclick events to the delete-icons in the bulk editors the post title list.
-		$('#bulk-titles').html(te);
+		// Populate the list of items to bulk edit.
+		$( '#bulk-titles' ).html( '<ul id="bulk-titles-list" role="list">' + te + '</ul>' );
+
 		/**
-		 * Binds on click events to the checkboxes before the posts in the table.
+		 * Binds on click events to handle the list of items to bulk edit.
 		 *
 		 * @listens click
 		 */
-		$('#bulk-titles a').click(function(){
-			var id = $(this).attr('id').substr(1);
+		$( '#bulk-titles .ntdelbutton' ).click( function() {
+			var $this = $( this ),
+				id = $this.attr( 'id' ).substr( 1 ),
+				$prev = $this.parent().prev().children( '.ntdelbutton' ),
+				$next = $this.parent().next().children( '.ntdelbutton' );
 
-			$('table.widefat input[value="' + id + '"]').prop('checked', false);
-			$('#ttle'+id).remove();
+			$( 'table.widefat input[value="' + id + '"]' ).prop( 'checked', false );
+			$( '#_' + id ).parent().remove();
+			wp.a11y.speak( wp.i18n.__( 'Item removed.' ), 'assertive' );
+
+			// Move focus to a proper place when items are removed.
+			if ( $next.length ) {
+				$next.focus();
+			} else if ( $prev.length ) {
+				$prev.focus();
+			} else {
+				$( '#bulk-titles-list' ).remove();
+				inlineEditPost.revert();
+				wp.a11y.speak( wp.i18n.__( 'All selected items have been removed. Select new items to use Bulk Actions.' ) );
+			}
 		});
 
 		// Enable auto-complete for tags when editing posts.
@@ -238,6 +260,8 @@ window.wp = window.wp || {};
 			} );
 		}
 
+		// Set initial focus on the Bulk Edit region.
+		$( '#bulk-edit .inline-edit-wrapper' ).attr( 'tabindex', '-1' ).focus();
 		// Scrolls to the top of the table where the editor is rendered.
 		$('html, body').animate( { scrollTop: 0 }, 'fast' );
 	},
@@ -249,7 +273,7 @@ window.wp = window.wp || {};
 	 *
 	 * @memberof inlineEditPost
 	 *
-	 * @param {number|Object} id The id of the clicked post or an element within a post
+	 * @param {number|Object} id The ID of the clicked post or an element within a post
 	 *                           table row.
 	 * @return {boolean} Always returns false at the end of execution.
 	 */
@@ -270,6 +294,10 @@ window.wp = window.wp || {};
 		editRow = $('#inline-edit').clone(true);
 		$( 'td', editRow ).attr( 'colspan', $( 'th:visible, td:visible', '.widefat:first thead' ).length );
 
+		// Remove the ID from the copied row and let the `for` attribute reference the hidden ID.
+		$( 'td', editRow ).find('#quick-edit-legend').removeAttr('id');
+		$( 'td', editRow ).find('p[id^="quick-edit-"]').removeAttr('id');
+
 		$(t.what+id).removeClass('is-expanded').hide().after(editRow).after('<tr class="hidden"></tr>');
 
 		// Populate fields in the quick edit window.
@@ -277,7 +305,7 @@ window.wp = window.wp || {};
 		if ( !$(':input[name="post_author"] option[value="' + $('.post_author', rowData).text() + '"]', editRow).val() ) {
 
 			// The post author no longer has edit capabilities, so we need to add them to the list of authors.
-			$(':input[name="post_author"]', editRow).prepend('<option value="' + $('.post_author', rowData).text() + '">' + $('#' + t.type + '-' + id + ' .author').text() + '</option>');
+			$(':input[name="post_author"]', editRow).prepend('<option value="' + $('.post_author', rowData).text() + '">' + $('#post-' + id + ' .author').text() + '</option>');
 		}
 		if ( $( ':input[name="post_author"] option', editRow ).length === 1 ) {
 			$('label.inline-edit-author', editRow).hide();
@@ -327,7 +355,12 @@ window.wp = window.wp || {};
 			var terms = $(this),
 				taxname = $(this).attr('id').replace('_' + id, ''),
 				textarea = $('textarea.tax_input_' + taxname, editRow),
-				comma = inlineEditL10n.comma;
+				comma = wp.i18n._x( ',', 'tag delimiter' ).trim();
+
+			// Ensure the textarea exists.
+			if ( ! textarea.length ) {
+				return;
+			}
 
 			terms.find( 'img' ).replaceWith( function() { return this.alt; } );
 			terms = terms.text();
@@ -343,9 +376,14 @@ window.wp = window.wp || {};
 		});
 
 		// Handle the post status.
+		var post_date_string = $(':input[name="aa"]').val() + '-' + $(':input[name="mm"]').val() + '-' + $(':input[name="jj"]').val();
+		post_date_string += ' ' + $(':input[name="hh"]').val() + ':' + $(':input[name="mn"]').val() + ':' + $(':input[name="ss"]').val();
+		var post_date = new Date( post_date_string );
 		status = $('._status', rowData).text();
-		if ( 'future' !== status ) {
+		if ( 'future' !== status && Date.now() > post_date ) {
 			$('select[name="_status"] option[value="future"]', editRow).remove();
+		} else {
+			$('select[name="_status"] option[value="publish"]', editRow).remove();
 		}
 
 		pw = $( '.inline-edit-password-input' ).prop( 'disabled', false );
@@ -378,20 +416,20 @@ window.wp = window.wp || {};
 		}
 
 		$(editRow).attr('id', 'edit-'+id).addClass('inline-editor').show();
-		$('.ptitle', editRow).focus();
+		$('.ptitle', editRow).trigger( 'focus' );
 
 		return false;
 	},
 
 	/**
 	 * Saves the changes made in the quick edit window to the post.
-	 * AJAX saving is only for Quick Edit and not for bulk edit.
+	 * Ajax saving is only for Quick Edit and not for bulk edit.
 	 *
 	 * @since 2.7.0
 	 *
-	 * @param  {int}     id The id for the post that has been changed.
-	 * @return {boolean}    False, so the form does not submit when pressing
-	 *                      Enter on a focused field.
+	 * @param {number} id The ID for the post that has been changed.
+	 * @return {boolean} False, so the form does not submit when pressing
+	 *                   Enter on a focused field.
 	 */
 	save : function(id) {
 		var params, fields, page = $('.post_status_page').val() || '';
@@ -411,6 +449,13 @@ window.wp = window.wp || {};
 		};
 
 		fields = $('#edit-'+id).find(':input').serialize();
+
+		var status = $(':input[name="_status"]').val();
+
+		if ( [ 'draft', 'pending', 'auto-draft' ].includes( status ) ) {
+			params.edit_date = 'false';
+		}
+
 		params = fields + '&' + $.param(params);
 
 		// Make Ajax request.
@@ -420,7 +465,6 @@ window.wp = window.wp || {};
 					$error = $errorNotice.find( '.error' );
 
 				$( 'table.widefat .spinner' ).removeClass( 'is-active' );
-				$( '.ac_results' ).hide();
 
 				if (r) {
 					if ( -1 !== r.indexOf( '<tr' ) ) {
@@ -430,8 +474,8 @@ window.wp = window.wp || {};
 							// Move focus back to the Quick Edit button. $( this ) is the row being animated.
 							$( this ).find( '.editinline' )
 								.attr( 'aria-expanded', 'false' )
-								.focus();
-							wp.a11y.speak( inlineEditL10n.saved );
+								.trigger( 'focus' );
+							wp.a11y.speak( wp.i18n.__( 'Changes saved.' ) );
 						});
 					} else {
 						r = r.replace( /<.[^<>]*?>/g, '' );
@@ -441,8 +485,8 @@ window.wp = window.wp || {};
 					}
 				} else {
 					$errorNotice.removeClass( 'hidden' );
-					$error.html( inlineEditL10n.error );
-					wp.a11y.speak( inlineEditL10n.error );
+					$error.text( wp.i18n.__( 'Error while saving the changes.' ) );
+					wp.a11y.speak( wp.i18n.__( 'Error while saving the changes.' ) );
 				}
 			},
 		'html');
@@ -466,7 +510,6 @@ window.wp = window.wp || {};
 
 		if ( id ) {
 			$( '.spinner', $tableWideFat ).removeClass( 'is-active' );
-			$( '.ac_results' ).hide();
 
 			if ( 'bulk-edit' === id ) {
 
@@ -478,7 +521,7 @@ window.wp = window.wp || {};
 				$('#inlineedit').append( $('#bulk-edit') );
 
 				// Move focus back to the Bulk Action button that was activated.
-				$( '#' + inlineEditPost.whichBulkButtonId ).focus();
+				$( '#' + inlineEditPost.whichBulkButtonId ).trigger( 'focus' );
 			} else {
 
 				// Remove both the inline-editor and its hidden tr siblings.
@@ -488,7 +531,7 @@ window.wp = window.wp || {};
 				// Show the post row and move focus back to the Quick Edit button.
 				$( this.what + id ).show().find( '.editinline' )
 					.attr( 'aria-expanded', 'false' )
-					.focus();
+					.trigger( 'focus' );
 			}
 		}
 
@@ -496,15 +539,15 @@ window.wp = window.wp || {};
 	},
 
 	/**
-	 * Gets the id for a the post that you want to quick edit from the row in the quick
+	 * Gets the ID for a the post that you want to quick edit from the row in the quick
 	 * edit table.
 	 *
 	 * @since 2.7.0
 	 *
 	 * @memberof inlineEditPost
 	 *
-	 * @param  {Object} o DOM row object to get the id for.
-	 * @return {string}   The post id extracted from the table row in the object.
+	 * @param {Object} o DOM row object to get the ID for.
+	 * @return {string} The post ID extracted from the table row in the object.
 	 */
 	getId : function(o) {
 		var id = $(o).closest('tr').attr('id'),
@@ -513,10 +556,16 @@ window.wp = window.wp || {};
 	}
 };
 
-$( document ).ready( function(){ inlineEditPost.init(); } );
+$( function() { inlineEditPost.init(); } );
 
 // Show/hide locks on posts.
-$( document ).on( 'heartbeat-tick.wp-check-locked-posts', function( e, data ) {
+$( function() {
+
+	// Set the heartbeat interval to 15 seconds.
+	if ( typeof wp !== 'undefined' && wp.heartbeat ) {
+		wp.heartbeat.interval( 15 );
+	}
+}).on( 'heartbeat-tick.wp-check-locked-posts', function( e, data ) {
 	var locked = data['wp-check-locked-posts'] || {};
 
 	$('#the-list tr').each( function(i, el) {
@@ -529,7 +578,14 @@ $( document ).on( 'heartbeat-tick.wp-check-locked-posts', function( e, data ) {
 				row.find('.check-column checkbox').prop('checked', false);
 
 				if ( lock_data.avatar_src ) {
-					avatar = $( '<img class="avatar avatar-18 photo" width="18" height="18" alt="" />' ).attr( 'src', lock_data.avatar_src.replace( /&amp;/g, '&' ) );
+					avatar = $( '<img />', {
+						'class': 'avatar avatar-18 photo',
+						width: 18,
+						height: 18,
+						alt: '',
+						src: lock_data.avatar_src,
+						srcset: lock_data.avatar_src_2x ? lock_data.avatar_src_2x + ' 2x' : undefined
+					} );
 					row.find('.column-title .locked-avatar').empty().append( avatar );
 				}
 				row.addClass('wp-locked');
@@ -550,12 +606,6 @@ $( document ).on( 'heartbeat-tick.wp-check-locked-posts', function( e, data ) {
 	if ( check.length ) {
 		data['wp-check-locked-posts'] = check;
 	}
-}).ready( function() {
-
-	// Set the heartbeat interval to 15 seconds.
-	if ( typeof wp !== 'undefined' && wp.heartbeat ) {
-		wp.heartbeat.interval( 15 );
-	}
 });
 
 })( jQuery, window.wp );
